@@ -1,7 +1,7 @@
 /*
  * dfu-programmer
  *
- * $Id: atmel.h 75 2008-10-31 08:19:42Z schmidtw $
+ * $Id: atmel.h 85 2009-04-30 09:46:27Z schmidtw $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,10 +54,27 @@ typedef struct {
     int16_t hsb;                // Hardware Security Byte
 } atmel_device_info_t;
 
+typedef struct {
+    int32_t lock;               // Locked region
+    int32_t epfl;               // External Privileged fetch lock
+    int32_t bootprot;           // Bootloader protected area
+    int32_t bodlevel;           // Brown-out detector trigger level
+    int32_t bodhyst;            // BOD hysteresis enable
+    int32_t boden;              // BOD enable state
+    int32_t isp_bod_en;         // Tells the ISP to enable BOD
+    int32_t isp_io_cond_en;     // ISP uses User page to launch bootloader
+    int32_t isp_force;          // Start the ISP no matter what
+} atmel_avr32_fuses_t;
+
 int32_t atmel_read_config( dfu_device_t *device,
                            atmel_device_info_t *info );
+int32_t atmel_read_fuses( dfu_device_t *device,
+                          atmel_avr32_fuses_t * info );
 int32_t atmel_erase_flash( dfu_device_t *device,
                            const uint8_t mode );
+int32_t atmel_set_fuse( dfu_device_t *device,
+                          const uint8_t property,
+                          const uint32_t value );
 int32_t atmel_set_config( dfu_device_t *device,
                           const uint8_t property,
                           const uint8_t value );
@@ -66,7 +83,8 @@ int32_t atmel_read_flash( dfu_device_t *device,
                           const uint32_t end,
                           uint8_t* buffer,
                           const size_t buffer_len,
-                          const dfu_bool eeprom );
+                          const dfu_bool eeprom,
+                          const dfu_bool user );
 int32_t atmel_blank_check( dfu_device_t *device,
                            const uint32_t start,
                            const uint32_t end );
@@ -77,6 +95,9 @@ int32_t atmel_flash( dfu_device_t *device,
                      const uint32_t end,
                      const size_t flash_page_size,
                      const dfu_bool eeprom );
+int32_t atmel_user( dfu_device_t *device,
+                    int16_t *buffer,
+                    const uint32_t end );
 int32_t atmel_start_app( dfu_device_t *device );
 
 void atmel_print_device_info( FILE *stream, atmel_device_info_t *info );
