@@ -1,7 +1,7 @@
 /*
  * dfu-programmer
  *
- * $Id: commands.h 48 2007-03-28 07:35:45Z schmidtw $
+ * $Id: commands.h 71 2008-10-02 05:48:41Z schmidtw $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,10 +21,10 @@
 #ifndef __COMMANDS_H__
 #define __COMMANDS_H__
 
-#include <usb.h>
+#include <stdint.h>
 #include "arguments.h"
+#include "dfu-device.h"
 
-int execute_command( struct usb_dev_handle *device,
-                     int interface,
-                     struct programmer_arguments *args );
+int32_t execute_command( dfu_device_t *device,
+                         struct programmer_arguments *args );
 #endif
