@@ -13,7 +13,7 @@
  * http://www.sbprojects.com
  * In the "Knowledge Base" section.
  *
- * $Id: intel_hex.c 76 2008-10-31 08:29:40Z schmidtw $
+ * $Id: intel_hex.c 96 2011-01-17 01:53:24Z schmidtw $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
 #include <stdio.h>
 #include <stdint.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "intel_hex.h"
 
@@ -127,13 +128,15 @@ static void intel_process_address( struct intel_record *record )
     switch( record->type ) {
         case 2:
             /* 0x1238 -> 0x00012380 */
-            record->address = (record->data[0] << 8) | record->data[1];
+            record->address = ((0xff & record->data[0]) << 8);
+            record->address |= (0xff & record->data[1]);
             record->address *= 16;
             break;
 
         case 4:
             /* 0x1234 -> 0x12340000 */
-            record->address = (record->data[0] << 8) | record->data[1];
+            record->address = ((0xff & record->data[0]) << 8);
+            record->address |= (0xff & record->data[1]);
             record->address <<= 16;
             break;
 
@@ -231,10 +234,14 @@ int16_t *intel_hex_to_buffer( char *filename, int max_size, int *usage )
         goto error;
     }
 
-    fp = fopen( filename, "r" );
-    if( NULL == fp ) {
-        fprintf( stderr, "Error opening the file.\n" );
-        goto error;
+    if( 0 == strcmp("STDIN",filename) ) {
+        fp = stdin;
+    } else {
+        fp = fopen( filename, "r" );
+        if( NULL == fp ) {
+            fprintf( stderr, "Error opening the file.\n" );
+            goto error;
+        }
     }
 
     memory = (int16_t *) malloc( max_size * sizeof(int16_t) );
